package org.processmining.big4prom.algorithms;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.in.XesXmlParser;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.out.XesXmlSerializer;
import org.processmining.contexts.uitopia.UIContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.PluginDescriptor;
import org.processmining.framework.plugin.PluginExecutionResult;
import org.processmining.framework.plugin.PluginParameterBinding;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetImpl;
import org.processmining.models.heuristics.HeuristicsNet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.InductiveMiner.mining.MiningParametersIMi;
import org.processmining.plugins.InductiveMiner.plugins.IM;
import org.processmining.plugins.astar.petrinet.PetrinetReplayerWithoutILP;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.heuristicsnet.miner.heuristics.converter.HeuristicsNetToPetriNetConverter;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.FlexibleHeuristicsMiner;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.settings.HeuristicsMinerSettings;
import org.processmining.plugins.log.logabstraction.LogRelations;
import org.processmining.plugins.petrinet.replayer.PNLogReplayer;
import org.processmining.plugins.petrinet.replayer.algorithms.costbasedcomplete.CostBasedCompleteParam;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.petrinet.replayresult.SwappedMove;
import org.processmining.plugins.petrinet.replayresult.ViolatingSyncMove;
import org.processmining.plugins.pnml.exporting.PnmlExportNetToPNML;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

import nl.tue.astar.AStarException;

//this class takes an event log, indications about process discovery technique, and output folder and generates:
//the Petri Net; the Conformance Checker Results; the folders needed to the IG algorithm
public class InputGenerator {
	private XLog log;
	private String outputFolder;
	private String pdTechnique;
	private String petriNetPath;
	//variables for replayer
	private NumberFormat nf = NumberFormat.getInstance();
	private static int RELIABLEMIN = 0;
	private static int RELIABLEMAX = 1;
	private static int MIN = 2;
	private static int MAX = 3;
	private static int SVAL = 4;
	private static int SVALRELIABLE = 5;
	private static int MVAL = 6;
	private static int MVALRELIABLE = 7;
	private static int PERFECTCASERELIABLECOUNTER = 8;
	private int numReliableSynchronized = 0;
	private int numReliableModelOnlyInvi = 0;
	private int numReliableModelOnlyReal = 0;
	private int numReliableLogOnly = 0;
	private int numReliableViolations = 0;
	private int numSwapViolations = 0;
	private int numReplaceViolations = 0;

	private int numCaseInvolved = 0;
	private int numReliableCaseInvolved;
	
	private PetrinetImpl petriNet;// = null;
	private PNRepResult replayerRes; // = null;
	private PluginContext plugContext;
	private UIContext mainContext;
	
	
	public InputGenerator(XLog log, /*String outputFolder,*/ String pdTechnique, PluginContext plugContext, UIContext mainContext) {
		super();
		this.log = log;
		//this.outputFolder = outputFolder;
		this.pdTechnique = pdTechnique;
		this.mainContext=mainContext;
		this.plugContext=plugContext;
	}
	
	
	public void generateInput(){
		// DA QUA
		// PetrinetImpl petriNet=null;
		File netFile=new File("provaPetri.pnml");
		PnmlExportNetToPNML exporter;
		XLogInfo logInfo =  XLogInfoFactory.createLogInfo(log);
		Collection<XEventClassifier> classifiers= logInfo.getEventClassifiers();
		Iterator<XEventClassifier> it= classifiers.iterator();
		XEventClassifier usedClassifier=null;
		boolean flag=false;
		String nameLegacyClassifier="";
		while(it.hasNext() && !flag){
			XEventClassifier classifier=it.next();
			if(classifier.toString().contains("Event Name")){
				usedClassifier=classifier;
				flag=true;
			}
		}
		//log=checkEventLog();
		//logInfo=XLogInfoFactory.createLogInfo(log);
		//log.getClassifiers().add(usedClassifier);
		switch(pdTechnique){
		case "Alpha":
			//the event log does not have to have a transiction life cycle, to avoid problems during conformance
			//checking
			//XLog originalLog=log;
			//checkEventLog();
			Object[] alphaRes=null;
			//AlphaMiner<XEventClass, ? extends AlphaClassicAbstraction<XEventClass>, ? extends AlphaMinerParameters> miner = AlphaMinerFactory.createAlphaMiner(context, log, classifier, parameters);
			// try the right way, namely let's invoke the plug-in manager
			// This is done, by asking the plugin manager for a plugin, that:
			// 1) It's a plugin (i.e. the annotation is Plugin.class),
			// 2) It returns PetriNet (i.e. one of the return types is PetriNet.class or any subclass thereof),
			// 3) It can be executed in a child of this context, which is of type context.getPluginContextType(),
			// 4) It can be executed on the given input (i.e. no extra input is needed, and all input is used),
			// 5) It accepts the input in any given order (i.e. not in the specified order),
			// 6) It does not have to be user visible
			// 7) It can use objects given by log and summary (i.e. types log.getClass() and summary.getClass()).
			SortedSet<PluginDescriptor> allPlugins= mainContext.getPluginManager().getAllPlugins();
			Iterator<PluginDescriptor> itPlug=allPlugins.iterator();
//			ArrayList<String> returns= new ArrayList<String>();
//			while(itPlug.hasNext()){
//				PluginDescriptor descriptor=itPlug.next();
//				java.util.List<Class<?>> returnTypes=descriptor.getReturnTypes();
//				for(Class<?> itemClass: returnTypes){
//					String className=itemClass.getName();
//					if(!returns.contains(itemClass))
//						returns.add(className);
//				}
//			}
//			for(String returnItem: returns)
//				System.out.println(returnItem);
			Collection<Pair<Integer, PluginParameterBinding>> plugins = mainContext.getPluginManager().find(Plugin.class,
					Petrinet.class, mainContext.getPluginContextType(), true, false, true, XLog.class,  logInfo.getClass());
			if (plugins.isEmpty()) {
				plugContext.log("No plugin found to derive Petri net, please specify the net manually",
						MessageLevel.ERROR);
				
			}
			else{
				int pluginNumber=plugins.size();
				Iterator<Pair<Integer, PluginParameterBinding>> pluginIt=plugins.iterator();
				while(pluginIt.hasNext() && pluginNumber>0){
					Pair<Integer, PluginParameterBinding> plugin = plugins.iterator().next();
					PluginParameterBinding pluginParB=plugin.getSecond();
					PluginDescriptor descriptor=pluginParB.getPlugin();
					String pluginName=descriptor.getName();
					if(pluginName.equals("Alpha Miner")){
						
						//LogRelations relations=obtainLogRelation();
						//java.util.List<Class<?>> parametersType=descriptor.getParameterTypes(1);
						// Now, the binding can be executed on the log and the summary
						// FIrst, we instantiate a new context for this plugin, which is a child context of the current context. 
						PluginContext c2 = plugContext.createChildContext("Alpha Miner");

						// Let's notify our lifecyclelisteners about the fact that we created a new context. this is
						// optional, but if this is not done, then the user interface doesn't show it (if there is a UI).
						plugContext.getPluginLifeCycleEventListeners().firePluginCreated(c2);
						// At this point, we execute the binding to get the PetriNet. For this, we call the invoke method
						// on the PluginParameterBinding stored in the plugin variable. The return type is PetriNet.class and
						// as input we give the new context c2, the log and the summary. Note that the plugin might return mulitple
						// objects, hence we extract the object with number x, where x is stored as the first element of the plugin
						// variable.
						PluginExecutionResult pluginResult = plugin.getSecond().invoke(c2,log, logInfo);
						try {
							pluginResult.synchronize();
						} catch (CancellationException | ExecutionException
								| InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						
						alphaRes = pluginResult.getResults();
						break;
					}
					//pluginNumber--;
				}
			}
			
			petriNet=(PetrinetImpl) alphaRes[0];
			Collection<Transition> transitions=petriNet.getTransitions();
			for(Transition t: transitions){
				t.getAttributeMap().put(AttributeMap.LABEL, t.getLabel().replace("+complete", ""));
			}
			//log=originalLog;
			break;
		case "Heuristic":
			while(it.hasNext()){
				XEventClassifier classifier=it.next();
				if(classifier.toString().equals("Event Name")){
					usedClassifier=classifier;
					break;
				}
			}
			HeuristicsMinerSettings settings =new HeuristicsMinerSettings();
			settings.setClassifier(usedClassifier);
			FlexibleHeuristicsMiner fhm= new FlexibleHeuristicsMiner(plugContext, log, logInfo, settings);
			HeuristicsNet heuristicNet= fhm.mine();
			Object[] heuRes=HeuristicsNetToPetriNetConverter.converter(plugContext, heuristicNet);
			petriNet=(PetrinetImpl) heuRes[0];
			//File netFile= new File("HeuristicPetriNet.pnml");
			//da rimettere Constants.MODEL=netFile.getAbsolutePath();
			exporter= new PnmlExportNetToPNML();
			try {
				exporter.exportPetriNetToPNMLFile(plugContext,petriNet, netFile);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
			break;
		case "Inductive":
			MiningParametersIMi parameters=new MiningParametersIMi();
			parameters.setClassifier(usedClassifier);
			Object[] indRes= IM.minePetriNet(plugContext, log, parameters);
			petriNet=(PetrinetImpl) indRes[0];
			
			/*
			netFile= new File(this.outputFolder+"InductivePetriNet.pnml");
			// da rimettere Constants.MODEL=netFile.getAbsolutePath();
			exporter= new PnmlExportNetToPNML();
			try {
				exporter.exportPetriNetToPNMLFile(plugContext,petriNet, netFile);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			System.out.println("res");
			*/
			
			break;
			
		}
		exporter= new PnmlExportNetToPNML();
		try {
			exporter.exportPetriNetToPNMLFile(plugContext,petriNet, netFile);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		PNLogReplayer replayer= new PNLogReplayer();
		// PNRepResult replayerRes=null;
		try {
			//create the mapping
			XEventClass dummyEventClass= new XEventClass("DUMMY", -1);
			TransEvClassMapping mapping=null;
			if(pdTechnique.contains("Alpha")){
				mapping= new TransEvClassMapping(XLogInfoImpl.STANDARD_CLASSIFIER, dummyEventClass);
				//mapping= new TransEvClassMapping(usedClassifier, dummyEventClass);
			}
			else{
				mapping= new TransEvClassMapping(usedClassifier, dummyEventClass);
			}
			mapEventsToTransition(mapping, petriNet, dummyEventClass);
			// create parameter
			CostBasedCompleteParam parameter = new CostBasedCompleteParam
					(this.getEventClasses(),
							dummyEventClass, petriNet.getTransitions(), 1, 1);
			parameter.setGUIMode(false);
			parameter.setCreateConn(false);
			Marking initialMarking=getInitialMarking(petriNet);
			Marking finalMarking=getFinalMarking(petriNet);
			parameter.setInitialMarking(initialMarking);
			parameter.setFinalMarkings(finalMarking);
			parameter.setMaxNumOfStates(200000);
			
			PetrinetReplayerWithoutILP selectedAlg = new PetrinetReplayerWithoutILP();
			this.replayerRes=replayer.replayLog(null, petriNet, this.log, mapping, 
					selectedAlg, parameter);
		
			 File repFile= new File("alignment.csv");
			 this.exportToFile(replayerRes,repFile, petriNet);
		} catch (AStarException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		/*
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		*/
		// PETRINET E PNET REPLAYER RESULT replayerRes
 catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
//	private Pnml importPnmlFromStream(PluginContext context, InputStream input, String filename, long fileSizeInBytes)
//					throws Exception {
//				/*
//				 * Get an XML pull parser.
//				 */
//				XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
//				factory.setNamespaceAware(true);
//				XmlPullParser xpp = factory.newPullParser();
//				/*
//				 * Initialize the parser on the provided input.
//				 */
//				xpp.setInput(input, null);
//				/*
//				 * Get the first event type.
//				 */
//				int eventType = xpp.getEventType();
//				/*
//				 * Create a fresh PNML object.
//				 */
//				Pnml pnml = new Pnml();
//
//				/*
//				 * Skip whatever we find until we've found a start tag.
//				 */
//				while (eventType != XmlPullParser.START_TAG) {
//					eventType = xpp.next();
//				}
//				/*
//				 * Check whether start tag corresponds to PNML start tag.
//				 */
//				if (xpp.getName().equals(Pnml.TAG)) {
//					/*
//					 * Yes it does. Import the PNML element.
//					 */
//					pnml.importElement(xpp, pnml);
//				} else {
//					/*
//					 * No it does not. Return null to signal failure.
//					 */
//					pnml.log(Pnml.TAG, xpp.getLineNumber(), "Expected pnml");
//				}
//				if (pnml.hasErrors()) {
//					context.getProvidedObjectManager().createProvidedObject("Log of PNML import", pnml.getLog(), XLog.class,
//							context);
//					return null;
//				}
//				return pnml;
//			}
		

	private XLog checkEventLog() {
		//remove the lifecycle attribute
		XLog newLog=log;
		Iterator<XTrace> it= newLog.iterator();
		while(it.hasNext()){
			XTrace trace= it.next();
			XAttributeMap traceAtt=trace.getAttributes();
			Iterator<XEvent> itEvent=trace.iterator();
			while(itEvent.hasNext()){
				XEvent event= itEvent.next();
				XAttributeMap attributesMap=event.getAttributes();
				String lifecycle="";
				try{
					lifecycle=attributesMap.get("lifecycle:transition").toString();
				}
				catch(NullPointerException e){
					System.out.println("lifecycle not present");
				}
				if(!lifecycle.equals(""))
					attributesMap.remove("lifecycle:transition");
				event.setAttributes(attributesMap);
								}
				}
		OutputStream output=null;
		try {
			output = new FileOutputStream("logChanged.xes");
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		XesXmlSerializer serializer= new XesXmlSerializer() ;
		try {
			serializer.serialize(newLog, output);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		newLog=null;
		XesXmlParser xesParser= new XesXmlParser();
		java.util.List<XLog> xlogList=null;
		try {
			xlogList = xesParser.parse(new File("logChanged.xes"));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
			newLog=xlogList.get(0);	
			return newLog;
		}
		
	


	private LogRelations obtainLogRelation() {
		//da tenerselo buono perchè funziona
//		PluginManager plugManager=mainContext.getPluginManager();
//		SortedSet<PluginDescriptor> pluginsSet=plugManager.getAllPlugins();
//		for(PluginDescriptor plugDescr: pluginsSet){
//			String namePlugin=plugDescr.getName();
//			if(namePlugin.contains("Relation"))
//			System.out.println("stop");
//		}
		XLogInfo summary=XLogInfoFactory.createLogInfo(log);
		Collection<Pair<Integer, PluginParameterBinding>> plugins = mainContext.getPluginManager().find(Plugin.class,
				LogRelations.class, mainContext.getPluginContextType(), true, false, false, XLog.class, summary.getClass());

		if (plugins.isEmpty()) {
			plugContext.log("No plugin found to create log relations, please specify relations manually",
					MessageLevel.ERROR);
			return null;
		}
		// Let's just take the first available plugin for the job of constructing log abstractions
		Pair<Integer, PluginParameterBinding> plugin = plugins.iterator().next();

		// Now, the binding can be executed on the log and the summary
		// FIrst, we instantiate a new context for this plugin, which is a child context of the current context. 
		PluginContext c2 = plugContext.createChildContext("Log Relation Constructor");

		// Let's notify our lifecyclelisteners about the fact that we created a new context. this is
		// optional, but if this is not done, then the user interface doesn't show it (if there is a UI).
		plugContext.getPluginLifeCycleEventListeners().firePluginCreated(c2);

		// At this point, we execute the binding to get the LogRelations. For this, we call the invoke method
		// on the PluginParameterBinding stored in the plugin variable. The return type is LogRelations.class and
		// as input we give the new context c2, the log and the summary. Note that the plugin might return mulitple
		// objects, hence we extract the object with number x, where x is stored as the first element of the plugin
		// variable.
		XLogInfo info = XLogInfoFactory.createLogInfo(log);
		PluginExecutionResult pluginResult = plugin.getSecond().invoke(c2, log, info);
		try {
			pluginResult.synchronize();
		} catch (CancellationException | ExecutionException
				| InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		LogRelations relations = pluginResult.<LogRelations>getResult(plugin.getFirst());
		return relations;
	}


	private void mapEventsToTransition(TransEvClassMapping mapping,
			PetrinetImpl petriNet, XEventClass dummy) {
		Collection<Transition> transitionList=petriNet.getTransitions();
		Collection<XEventClass> eventClasses=this.getEventClasses();
		ArrayList<Transition> unmapped= new ArrayList<Transition>();
		for(Transition t: transitionList){
			boolean mapped=false;
			String tName=t.getLabel();
			for(XEventClass evClass: eventClasses){
				if(evClass.getId() != null && evClass.getId().equals(tName))
				{
					mapping.put(t, evClass);
					mapped=true;
					break;
				}
			}
			if(!mapped)
				unmapped.add(t);
		}
		for(Transition t: unmapped){
			mapping.put(t, dummy);
		}
		
	}

	private Collection<XEventClass> getEventClasses() {
		if(pdTechnique.contains("Alpha")){
			XEventClasses eventClassesInfo= XLogInfoFactory.createLogInfo(log).getEventClasses();
			return eventClassesInfo.getClasses();
		}
		else{
		XLogInfo info=null;
		if(this.log.getClassifiers().size()>0)
			info=this.log.getInfo(this.log.getClassifiers().get(0));
		Collection<XEventClass> eventClasses=null;
		if(info!=null)
			eventClasses=info.getEventClasses().getClasses();
		else{
			int cont=1;
			eventClasses= new ArrayList<XEventClass>();
			for(XTrace trace: this.log){
				for(XEvent event: trace){
					String eventClassName=event.getAttributes().get("concept:name").toString();
					XEventClass eventClass= new XEventClass(eventClassName, cont);
					if(!eventClasses.contains(eventClass)){
						eventClasses.add(eventClass);
						cont++;
					}
					
				}
			}
			
		}
		return eventClasses;
		}
	}

	private Marking getFinalMarking(PetrinetImpl petriNet) {
		Collection<Place> places=petriNet.getPlaces();
		Collection<Place> endMarkingPlaces= new ArrayList<Place>();
		for(Place place: places){
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outputEdges=petriNet.getOutEdges(place);
			if(outputEdges==null || outputEdges.size()==0){
				endMarkingPlaces.add(place);
				break;
			}
		}
		Marking finMarking= new Marking(endMarkingPlaces);
		return finMarking;
	}

	private Marking getInitialMarking(PetrinetImpl petriNet) {
		Collection<Place> places=petriNet.getPlaces();
		Collection<Place> initMarkingPlaces= new ArrayList<Place>();
		for(Place place: places){
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inputEdges=petriNet.getInEdges(place);
			if(inputEdges==null || inputEdges.size()==0){
				initMarkingPlaces.add(place);
				break;
			}
		}
		Marking initMarking= new Marking(initMarkingPlaces);
		return initMarking;
	}
	
	public void exportToFile( PNRepResult repResult, File file, PetrinetImpl petriNet) throws IOException {
	System.gc();
	nf.setMinimumFractionDigits(2);
	nf.setMaximumFractionDigits(2);


	// export  to file
	BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
	bw.write(createStringRep(repResult, petriNet));
	bw.close();

}

public String createStringRep(PNRepResult repResult, PetrinetGraph net) {
	String newLineChar = "\n";
	char commaSeparator = ',';
	XConceptExtension ce = XConceptExtension.instance();

	StringBuilder sb = new StringBuilder();
	sb.append("Result of replaying ");
	sb.append(net.getLabel());
	sb.append(" on ");
	sb.append(ce.extractName(log));
	sb.append(newLineChar);
	sb.append(newLineChar);

	sb.append("Index" + commaSeparator + "Case IDs" + commaSeparator + "NumOfCases" + commaSeparator + "IsReliable"
			+ commaSeparator);

	Iterator<SyncReplayResult> it = repResult.iterator();
	SyncReplayResult firstReplayResult = it.next();

	if (firstReplayResult != null) {
		// information for later
		Map<String, Double[]> calculations = new HashMap<String, Double[]>();

		// get all information exist in replay result
		Map<String, Double> infos = firstReplayResult.getInfo();
		Set<String> infoKeys = infos.keySet();
		String[] infoKeysArr = infoKeys.toArray(new String[infoKeys.size()]);
		for (String info : infoKeysArr) {
			sb.append("\"");
			sb.append(info);
			sb.append("\"");
			sb.append(commaSeparator);
		}

		sb.append("Match");
		sb.append(newLineChar);

		// now, create table content
		int indexCounter = 1;

		sb.append(indexCounter);
		sb.append(commaSeparator);

		String separatorStr = "";
		sb.append("\"");
		for (Integer index : firstReplayResult.getTraceIndex()) {
			sb.append(separatorStr);
			sb.append(ce.extractName(log.get(index)));
			separatorStr = "|";
		}
		sb.append("\"");

		sb.append(commaSeparator);

		sb.append(firstReplayResult.getTraceIndex().size());
		sb.append(commaSeparator);

		sb.append(firstReplayResult.isReliable() ? "Yes" : "No");
		sb.append(commaSeparator);

		for (String infoKey : infoKeysArr) {
			Double val = infos.get(infoKey);
			sb.append("\"");
			sb.append(nf.format(val));
			sb.append("\"");
			updateCalculations(firstReplayResult, infoKey, calculations, firstReplayResult.getTraceIndex().size(),
					val);
			sb.append(commaSeparator);
		}

		sb.append(generateMatchString(firstReplayResult));
		sb.append(newLineChar);

		indexCounter++;

		// add stats for first result
		numCaseInvolved += firstReplayResult.getTraceIndex().size();
		if (firstReplayResult.isReliable()) {
			numReliableCaseInvolved += numCaseInvolved;
		}

		while (it.hasNext()) {
			SyncReplayResult syncRepResult = it.next();
			int caseIDSize = syncRepResult.getTraceIndex().size();

			sb.append(indexCounter);
			sb.append(commaSeparator);

			separatorStr = "";
			sb.append("\"");
			for (Integer index : syncRepResult.getTraceIndex()) {
				sb.append(separatorStr);
				sb.append(ce.extractName(log.get(index)));
				separatorStr = "|";
			}
			sb.append("\"");
			sb.append(commaSeparator);

			sb.append(caseIDSize);
			sb.append(commaSeparator);

			sb.append(syncRepResult.isReliable() ? "Yes" : "No");
			sb.append(commaSeparator);

			Map<String, Double> allInfo = syncRepResult.getInfo();
			for (String infoKey : infoKeysArr) {
				Double val = allInfo.get(infoKey);
				sb.append("\"");
				sb.append(nf.format(val));
				sb.append("\"");
				updateCalculations(syncRepResult, infoKey, calculations, caseIDSize, val);
				sb.append(commaSeparator);
			}

			sb.append(generateMatchString(syncRepResult));
			sb.append(newLineChar);

			indexCounter++;

			// add stats
			numCaseInvolved += caseIDSize;
			if (syncRepResult.isReliable()) {
				numReliableCaseInvolved += caseIDSize;
			}

		}

		sb.append(newLineChar);
		sb.append(newLineChar);

		sb.append("ONLY RELIABLE RESULTS");

		sb.append(newLineChar);
		sb.append(newLineChar);

		// update based on calculations
		sb.append("Property");
		sb.append(commaSeparator);
		sb.append("Value");
		sb.append(newLineChar);

		sb.append("#Reliable cases replayed");
		sb.append(commaSeparator);
		sb.append(numReliableCaseInvolved);
		sb.append(newLineChar);

		sb.append("#Synchronous ev.class (log+model)");
		sb.append(commaSeparator);
		sb.append(numReliableSynchronized);
		sb.append(newLineChar);

		sb.append("#Skipped ev.class");
		sb.append(commaSeparator);
		sb.append(numReliableModelOnlyReal);
		sb.append(newLineChar);

		sb.append("#Unobservable ev.class");
		sb.append(commaSeparator);
		sb.append(numReliableModelOnlyInvi);
		sb.append(newLineChar);

		sb.append("#Inserted ev.class");
		sb.append(commaSeparator);
		sb.append(numReliableLogOnly);
		sb.append(newLineChar);
		
		if (numSwapViolations > 0){
			sb.append("#Swapped Violations");
			sb.append(commaSeparator);
			sb.append(Integer.valueOf(numSwapViolations));
			sb.append(newLineChar);
		}
		
		if (numReplaceViolations > 0){
			sb.append("#Replacements Violations");
			sb.append(commaSeparator);
			sb.append(Integer.valueOf(numReplaceViolations));
			sb.append(newLineChar);
		}

		Map<String, Object> info = repResult.getInfo();
		if (info != null) {
			for (String key : info.keySet()) {
				sb.append(key);
				sb.append(commaSeparator);
				sb.append(info.get(key));
				sb.append(newLineChar);
			}
		}

		sb.append(newLineChar);
		sb.append(newLineChar);

		// update based on calculations
		sb.append("Property");
		sb.append(commaSeparator);
		sb.append("Average");
		sb.append(commaSeparator);
		sb.append("Minimum");
		sb.append(commaSeparator);
		sb.append("Maximum");
		sb.append(commaSeparator);
		sb.append("Std. Deviation");
		sb.append(commaSeparator);
		sb.append("Number of cases with value 1.00");
		sb.append(newLineChar);

		// property based, only reliable results
		for (String property : calculations.keySet()) {
			Double[] values = calculations.get(property);
			sb.append("\"");
			sb.append(property);
			sb.append("\"");
			sb.append(commaSeparator);

			sb.append("\"");
			sb.append(numReliableCaseInvolved == 0 ? "<NaN>" : nf.format(values[MVALRELIABLE]));
			sb.append("\"");
			sb.append(commaSeparator);

			sb.append("\"");
			sb.append(numReliableCaseInvolved == 0 ? "<NaN>" : nf.format(values[RELIABLEMIN]));
			sb.append("\"");
			sb.append(commaSeparator);

			sb.append("\"");
			sb.append(numReliableCaseInvolved == 0 ? "<NaN>" : nf.format(values[RELIABLEMAX]));
			sb.append("\"");
			sb.append(commaSeparator);

			double reliabledev = numReliableCaseInvolved == 0 ? 0 : Math.sqrt(values[SVALRELIABLE]
					/ (numReliableCaseInvolved - 1));
			sb.append("\"");
			sb.append(Double.compare(reliabledev, Double.NaN) == 0 ? "<NaN>" : nf.format(reliabledev));
			sb.append("\"");
			sb.append(commaSeparator);

			sb.append("\"");
			sb.append(nf.format(values[PERFECTCASERELIABLECOUNTER]));
			sb.append("\"");
			sb.append(newLineChar);
		}

	}
	return sb.toString();
}

private void updateCalculations(SyncReplayResult res, String property, Map<String, Double[]> calculations,
		int caseIDSize, double value) {
	// use it to calculate property
	Double[] oldValues = calculations.get(property);
	if (oldValues == null) {
		oldValues = new Double[] { Double.MAX_VALUE, Double.MIN_VALUE, Double.MAX_VALUE, Double.MIN_VALUE, 0.00000,
				0.00000, 0.00000, 0.00000, 0.00000 };
		calculations.put(property, oldValues);
	}

	if (Double.compare(oldValues[MIN], value) > 0) {
		oldValues[MIN] = value;
	}
	if (Double.compare(oldValues[MAX], value) < 0) {
		oldValues[MAX] = value;
	}

	int counterCaseIDSize = 0;
	if (numCaseInvolved == 0) {
		oldValues[MVAL] = value;
		oldValues[SVAL] = 0.0000;
		counterCaseIDSize++;
	}
	for (int i = counterCaseIDSize; i < caseIDSize; i++) {
		double oldMVal = oldValues[MVAL];
		// AA: previous average calculation is wrong. Fix this
		// oldValues[MVAL] += ((value - oldValues[MVAL]) / (i + numCaseInvolved));
		oldValues[MVAL] += ((value - oldValues[MVAL]) / (i + numCaseInvolved + 1));
		oldValues[SVAL] += ((value - oldMVal) * (value - oldValues[MVAL]));
	}

	if (res.isReliable()) {
		if (Double.compare(oldValues[RELIABLEMIN], value) > 0) {
			oldValues[RELIABLEMIN] = value;
		}
		if (Double.compare(oldValues[RELIABLEMAX], value) < 0) {
			oldValues[RELIABLEMAX] = value;
		}

		counterCaseIDSize = 0;
		if (numReliableCaseInvolved == 0) {
			oldValues[MVALRELIABLE] = value;
			oldValues[SVALRELIABLE] = 0.0000;
			counterCaseIDSize++;
		}
		for (int i = counterCaseIDSize; i < caseIDSize; i++) {
			double oldMVal = oldValues[MVALRELIABLE];
			// AA: previous average calculation is wrong. Fix this
			// oldValues[MVALRELIABLE] += ((value - oldValues[MVALRELIABLE]) / (i + numReliableCaseInvolved));
			oldValues[MVALRELIABLE] += ((value - oldValues[MVALRELIABLE]) / (i + numReliableCaseInvolved + 1));
			oldValues[SVALRELIABLE] += ((value - oldMVal) * (value - oldValues[MVALRELIABLE]));
		}

		if (Double.compare(value, 1.0000000) == 0) {
			oldValues[PERFECTCASERELIABLECOUNTER] += caseIDSize;
		}
	}
}

private Object generateMatchString(SyncReplayResult res) {
	StringBuilder sb = new StringBuilder();
	String limiter = "";
	java.util.List<Object> nodeInstance = res.getNodeInstance();

	int counter = 0;
	int size = res.getTraceIndex().size();
	for (StepTypes stepType : res.getStepTypes()) {
		sb.append(limiter);
		switch (stepType) {
			case L :
				sb.append("[L]");
				sb.append(nodeInstance.get(counter));
				if (res.isReliable()) {
					numReliableLogOnly += size;
				}
				;
				break;
			case MINVI :
				sb.append("[M-INVI]");
				sb.append(((Transition) nodeInstance.get(counter)).getLabel());
				if (res.isReliable()) {
					numReliableModelOnlyInvi += size;
				}
				;
				break;
			case MREAL :
				sb.append("[M-REAL]");
				sb.append(((Transition) nodeInstance.get(counter)).getLabel());
				if (res.isReliable()) {
					numReliableModelOnlyReal += size;
				}
				;
				break;
			case LMNOGOOD :
				sb.append("[L/M-NOGOOD]");
				sb.append(((Transition) nodeInstance.get(counter)).getLabel());
				if (res.isReliable()) {
					numReliableViolations += size;
				}
				;
				break;
			case LMREPLACED :
				sb.append("[L/M-REPLACED]");
				ViolatingSyncMove violation = (ViolatingSyncMove) nodeInstance.get(counter);
				sb.append("Trans:" + violation.getTransition() + "-->" + violation.getEventClass().getId());
				if (res.isReliable()) {
					numReplaceViolations += size;
				}
				break;
			case LMSWAPPED :
				sb.append("[L/M-SWAPPED]");
				SwappedMove swapMove = (SwappedMove) nodeInstance.get(counter);
				sb.append("Trans:" + swapMove.getOccurTrans().getLabel() + " insteadOf " + swapMove.getInsteadOf().getLabel());
				if (res.isReliable()) {
					numSwapViolations += size;
				}
				break;
			default :
				sb.append("[L/M]");
				sb.append(((Transition) nodeInstance.get(counter)).getLabel());
				if (res.isReliable()) {
					numReliableSynchronized += size;
				}
				;
		}
		limiter = "|";
		counter++;
	}
	return sb.toString();
}

	public XLog getLog() {
		return log;
	}
	public void setLog(XLog log) {
		this.log = log;
	}
	public String getOutputFolder() {
		return outputFolder;
	}
	public void setOutputFolder(String outputFolder) {
		this.outputFolder = outputFolder;
	}
	public String getPdTechnique() {
		return pdTechnique;
	}
	public void setPdTechnique(String pdTechnique) {
		this.pdTechnique = pdTechnique;
	}

	public String getPetriNetPath() {
		// TODO Auto-generated method stub
		return this.petriNetPath;
		
	}
	
	public PetrinetImpl getpetriNet()	{
		return this.petriNet;
		
	}
	
	public PNRepResult getPNRep(){
		return this.replayerRes;
	}
	   
	
}
