package org.processmining.big4prom.dialogs;

import instancegraph.gui.IPanelManager;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Array;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.processmining.framework.util.Pair;

public abstract class AbstractSettingsDialog extends JPanel
{
	private static final long serialVersionUID = 1073816526950228833L;

	protected Dimension preferredSize = new Dimension(100, 20);
	
	protected interface FloatChanged
	{
		public void update(float newValue);
	}

	protected interface IntChanged
	{
		public void update(int newValue);
	}
	
	protected interface BooleanChanged
	{
		public void update(boolean newValue);
	}
	
	protected interface SelectionChanged<T>
	{
		public void update(T newValue);
	}
	
	protected interface MultipleSelectionChanged<T>
	{
		public void update(int[] indices, T[] newValues);
	}
	
	protected JSlider horizontalSliderInt(IPanelManager manager, String name, 
			int val, int min, int max, 
			final IntChanged eventListener)
	{
		final JSlider slider = new JSlider(JSlider.HORIZONTAL, min, max, val);
		slider.setMinorTickSpacing((max - min) / 100);
		slider.setMajorTickSpacing((max - min) / 10);
		slider.setPaintTicks(true);
		slider.setPaintLabels(true);
		
		slider.addChangeListener(new ChangeListener()
		{
			public void stateChanged(ChangeEvent paramChangeEvent)
			{
				eventListener.update(slider.getValue());
			}
		});
		
		Dimension d = new Dimension(preferredSize);
		d.setSize(d.getWidth(), d.getHeight() * 2); // for labels and ticks
		slider.setPreferredSize(d);
		if (name != null) {
			manager.add(new JLabel(name));
		}
		manager.add(slider);
		return slider;
	}
	
	
	protected JSpinner spinnerFloat(IPanelManager manager, String name, 
			float val, float min, float max, float step, 
			final FloatChanged eventListener)
	{
		final SpinnerNumberModel model = new SpinnerNumberModel(
			val, min, max, step
		);
		JSpinner spinner = new JSpinner(model);
		spinner.addChangeListener(new ChangeListener()
		{
			public void stateChanged(ChangeEvent paramChangeEvent)
			{
				eventListener.update(model.getNumber().floatValue());
			}
		});
		spinner.setPreferredSize(new Dimension(preferredSize));
		if (name != null) {
			manager.add(new JLabel(name));
		}
		manager.add(spinner);
		return spinner;
	}

	protected JSpinner spinnerInt(IPanelManager manager, String name, 
			int val, int min, int max, int step, 
			final IntChanged eventListener)
	{
		final SpinnerNumberModel model = new SpinnerNumberModel(
			val, min, max, step
		);
		JSpinner spinner = new JSpinner(model);
		spinner.addChangeListener(new ChangeListener()
		{
			public void stateChanged(ChangeEvent paramChangeEvent)
			{
				eventListener.update(model.getNumber().intValue());
			}
		});
		spinner.setPreferredSize(new Dimension(preferredSize));
		if (name != null) {
			manager.add(new JLabel(name));
		}
		manager.add(spinner);
		return spinner;
	}

	protected JCheckBox checkboxBool(IPanelManager manager, String name, 
		boolean value, String checkLabel,
		final BooleanChanged eventListener)
	{
		final JCheckBox checkbox = new JCheckBox(checkLabel);
		checkbox.setSelected(value);
		checkbox.addChangeListener(new ChangeListener()
		{
			public void stateChanged(ChangeEvent paramChangeEvent)
			{
				eventListener.update(checkbox.isSelected());
			}
		});
		checkbox.setPreferredSize(new Dimension(preferredSize));
		if (name != null) {
			manager.add(new JLabel(name));
		}
		manager.add(checkbox);
		return checkbox;
	}
	
	protected <T> JComboBox<T> dropdown(IPanelManager manager, String name,
		T value, T[] options,
		final SelectionChanged<T> eventListener)
	{
		final JComboBox<T> combobox = new JComboBox<T>(options);
		combobox.setSelectedItem(value);
		combobox.addActionListener(new ActionListener()
		{
			@SuppressWarnings("unchecked")
			public void actionPerformed(ActionEvent e)
			{
				eventListener.update((T) combobox.getSelectedItem());
			}
		});
		
		combobox.setPreferredSize(new Dimension(preferredSize));
		if (name != null) {
			manager.add(new JLabel(name));
		}
		manager.add(combobox);
		return combobox;
	}
	
	protected <T> Pair<JList<T>, JScrollPane> list(IPanelManager manager, String name,
		T[] values, final T[] options,
		final MultipleSelectionChanged<T> eventListener)
	{
		int[] indices = new int[values.length];
		for (int i = 0; i < values.length; i++)
		{
			for (int j = 0; j < options.length; j++)
			{
				if (options[j].equals(values[i]))
				{
					indices[i] = j;
				}
			}
		}
		
		final JList<T> list = new JList<T>(options);
		list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		list.setSelectedIndices(indices);
		
		list.addListSelectionListener(new ListSelectionListener()
		{
			@SuppressWarnings("unchecked")
			public void valueChanged(ListSelectionEvent e)
			{
				if (!e.getValueIsAdjusting())
				{
					int[] indices = list.getSelectedIndices();
					T[] newValues = (T[]) Array.newInstance(options.getClass().getComponentType(), indices.length);
					for (int i = 0; i < indices.length; i++)
					{
						newValues[i] = options[indices[i]];
					}
					
					eventListener.update(indices, newValues);
				}
			}
		});
		
		//list.setPreferredSize(new Dimension(preferredSize));
		if (name != null) {
			manager.add(new JLabel(name));
		}

        JScrollPane scrollPane = new JScrollPane();
        scrollPane.setViewportView(list);
		manager.add(scrollPane);
		
		return new Pair<JList<T>, JScrollPane>(list, scrollPane);
	}
}
