package org.processmining.big4prom.plugins;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Set;

import org.jgrapht.DirectedGraph;
import org.jgrapht.graph.DefaultEdge;
import org.processmining.big4prom.models.ModelResult;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Export IG set as DOT (.dot)", returnLabels = {}, returnTypes = {}, parameterLabels = {
		"Instance Graphs Set", "File" }, userAccessible = true)
@UIExportPlugin(description = "Export IG set", extension = "dot")
public class ExportIGSetAsDot {
	@PluginVariant(variantLabel = "DOT (.dot)", requiredParameterLabels = { 0, 1 })
	
	public void exportIGSet(PluginContext context,ModelResult res, File file){
		HashMap<Integer, DirectedGraph<String, DefaultEdge>> graphMap=res.getRes();
		for(int i=1; i<res.getRes().size()+1;i++){
		DirectedGraph<String, DefaultEdge> graph1=graphMap.get(i);
		FileWriter writer = null;
		try {
			String filePath=file.getAbsolutePath().substring(0, file.getAbsolutePath().lastIndexOf("."))+i+".dot";
			writer = new FileWriter(filePath, false);
			writer.write("digraph G { \n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Set<String> posVertexes = graph1.vertexSet();
		for (String posVertex : posVertexes) {
			
					try {
						if(posVertex.contains("-")){
							String[] stringTokens=posVertex.split("-");
							String pos= stringTokens[0];
							String label=stringTokens[1];
							writer.write(pos + " [label=\""
									+ label
									+ "\",color=black,fontcolor=black];\n");
						}
						else{
						writer.write(posVertex + " [label=\""
								+ posVertex
								+ "\",color=black,fontcolor=black];\n");
						}
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				
			
		}
		Set<DefaultEdge> posEdges = graph1.edgeSet();
		for (DefaultEdge posEdge : posEdges) {
			//String edgeLabel="[label=\""+edgeLabel+"\",color=black,fontcolor=black]";
			try {
				String edgeLabel="";
				String source=graph1.getEdgeSource(posEdge);
				String target= graph1.getEdgeTarget(posEdge);
				if(source.contains("-")){
					String[] sourceTokens=source.split("-");
					String posSource=sourceTokens[0];
					String[] targetTokens=target.split("-");
					String posTarget=targetTokens[0];
					writer.write(posSource
							+ "->" + posTarget +" "+edgeLabel
							+ ";\n");
				}
				else{
				writer.write(graph1.getEdgeSource(posEdge)
						+ "->" + graph1.getEdgeTarget(posEdge) +" "+edgeLabel
						+ "\n;");
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		try {
			writer.write("}");
			writer.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	}
	
}
