package org.processmining.big4prom.plugins;

import instancegraph.AlignedTrace;
import instancegraph.DBManager;
import instancegraph.Event;
import instancegraph.IGBuilderUpdated;
import instancegraph.PetriNetParser;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.deckfour.xes.model.impl.XLogImpl;
import org.deckfour.xes.out.XesXmlSerializer;
import org.jgrapht.DirectedGraph;
import org.processmining.big4prom.algorithms.InputGenerator;
import org.processmining.big4prom.algorithms.InstanceGraphPluginAlgorithm;
import org.processmining.big4prom.dialogs.CheckBoxListener;
import org.processmining.big4prom.models.ModelResult;
import org.processmining.big4prom.parameters.IGConfiguration;
import org.processmining.contexts.uitopia.UIContext;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.ResetArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetImpl;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.petrinet.replayresult.SwappedMove;
import org.processmining.plugins.petrinet.replayresult.ViolatingSyncMove;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;



//import com.sun.java.util.jar.pack.Package.File;

@Plugin(name = "BIG4ProM", parameterLabels = { "Log", "PetriNet",
		"Replay result" }, returnLabels = { "Instance Graphs Set" }, returnTypes = { ModelResult.class }, userAccessible = true, help = "Generates Instance Graphs From an event log")
public class InstanceGraphPlugin extends InstanceGraphPluginAlgorithm{

	// selected process discovery technique
	static String selectedPrcessDiscoveryMode;

	// flag checking the caller function
	static boolean checkCaller = false;
			
	static IGConfiguration config = new IGConfiguration(true); //M.
	
	// v1
	@UITopiaVariant(affiliation = "Università Politecnica delle Marche", author = "Laura Genga, Marco Cameranesi", email = "{l.genga, m.cameranesi}@univpm.it")
	@PluginVariant(requiredParameterLabels = { 0 })
	public static ModelResult instancegraph(UIPluginContext uiContext, XLog log) {
		UIContext mainContext=uiContext.getGlobalContext();
		PluginContext plugContext=uiContext.getParentContext();
		checkCaller = true;
		
		//IGConfiguration config = new IGConfiguration(true); // M.
		config = setConfig(uiContext, config, 1);
		String pdDiscovery=config.getpDiscovery();
		if(pdDiscovery.equals("Alpha"))
			config.setRep(false);
		if (!config.isRep()) {
			System.out.println("No rep");
		}
		System.out.println(pdDiscovery);
		// the event log has to be preprocessed to remove the possible lifecycle attribute --> to do
		XLog preprocessedLog=manipulateEventLog(log);
		InputGenerator iGen = new InputGenerator(preprocessedLog, pdDiscovery,plugContext,mainContext);
		iGen.generateInput();
		PetrinetImpl petriNet = iGen.getpetriNet();
		PNRepResult replayerRes = iGen.getPNRep();
				
		ModelResult mr = instancegraph(uiContext, preprocessedLog, petriNet, replayerRes);
		checkCaller = false;
		return mr;
	}

	// v2
	@UITopiaVariant(affiliation = "Università Politecnica delle Marche", author = "Laura Genga, Marco Cameranesi", email = "{l.genga, m.cameranesi}@univpm.it")
	@PluginVariant(requiredParameterLabels = { 0, 1, 2 })
	public static ModelResult instancegraph(UIPluginContext uiContext, XLog log, PetrinetGraph petriNet, PNRepResult pnrResult) 
	{
		uiContext.getProgress().setIndeterminate(true);
		if(checkCaller == false)
		{
			IGConfiguration config = new IGConfiguration(true);
			config = setConfig(uiContext, config, 2);
		}
		
		if (!config.isRep()) {
			System.out.println("No rep");
		}
		
		// ALIGNMENT
		Iterator<SyncReplayResult> it = pnrResult.iterator();
		ArrayList<ArrayList<String>> alignmentTable = new ArrayList<ArrayList<String>>();

		while (it.hasNext()) {
			SyncReplayResult res = it.next();
			String alignmentSt = generateMatchString(res).toString();
			XConceptExtension xc = XConceptExtension.instance();
			Iterator<Integer> it2 = res.getTraceIndex().iterator();

			while (it2.hasNext()) {
				int element = it2.next();
				String id = xc.extractName(log.get(element));
				ArrayList<String> elem = new ArrayList<String>();

				elem.add(id);
				elem.add(alignmentSt);
				alignmentTable.add(elem);
			}
		}

		// RETE DI PETRI
		Collection<Place> placesList = petriNet.getPlaces();
		ArrayList<ArrayList<String>> table = new ArrayList<ArrayList<String>>();

		for (Place place : placesList) {

			ArrayList<String> inTrLabel = new ArrayList<String>();
			ArrayList<String> outTrLabel = new ArrayList<String>();

			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges = petriNet
					.getOutEdges(place);
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges = petriNet
					.getInEdges(place);

			// get out transitions
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> outEdge : outEdges) {
				if (!(outEdge instanceof ResetArc)) {

					String outLabel = outEdge.getTarget().getLabel();
					String outID = outEdge.getTarget().getId().toString();

					if(outLabel.contains("tau")){
						Transition node=(Transition) outEdge.getTarget();
						if(node.isInvisible()){
							outLabel = outLabel + "tauINVISIBLE" + outID;
						}
						
					}

					outTrLabel.add(outLabel);
				}
			}

			// get in transitions
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> inEdge : inEdges) {

				String label = inEdge.getSource().getLabel();
				String inID = inEdge.getSource().getId().toString();

				if (label.contains("tau")) {
					Transition node=(Transition) inEdge.getSource();
					if(node.isInvisible())
						label = label + "tauINVISIBLE" + inID;
				}

				inTrLabel.add(label);
			}

			// creo i nuovi elementi da inserire nella tabella
			// SE esistono almeno un ingresso ed un'uscita
			if (inTrLabel.size() > 0 && outTrLabel.size() > 0) {
				for (int i = 0; i < inTrLabel.size(); i++) {
					for (int j = 0; j < outTrLabel.size(); j++) {
						ArrayList<String> element = new ArrayList<String>();
						element.add(inTrLabel.get(i));
						element.add(outTrLabel.get(j));
						table.add(element);
					}
				}
			}
		}
		
		// inserimento dati nel db
		DBManager db = new DBManager("jdbc:mysql://localhost/", "exampledb", "root", "");

		// cancellazione eventuali dati esistenti all'interno delle tabelle
		String emptyTable = "TRUNCATE alignedevent";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE alignments";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE anomalies";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE anomedges";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE causalrel";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE causalreltau";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE disconnected";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE edge";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE edges";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE event";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE nodes";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE place";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE startend";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE traceid";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE traceiddisconnected";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE traceidwabo";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE tracesgraph";
		db.queryUpdate(emptyTable);
		emptyTable = "TRUNCATE transition";
		db.queryUpdate(emptyTable);

		//
		PetriNetParser parser=null;

		ArrayList<AlignedTrace> conformanceResult = updateAlignment(alignmentTable, db);
		updateCasualRel(table, db);
		// deletion of tau relation
		deleteTauFromRel(0, 0, db);
		updateEventTable(log, db);

		HashMap<Integer, DirectedGraph<String, org.jgrapht.graph.DefaultEdge>> model = new HashMap<Integer, DirectedGraph<String, org.jgrapht.graph.DefaultEdge>>();

		Integer logSize = log.size();
		//IGBuilder igbuilder = new IGBuilder(log, db, conformanceResult, 0, logSize);
		boolean repFlag = config.isRep();
		
		// usando il seguente costruttore non funziona la variante cui si passa un solo parametro
		// ci sono problemi riguardanti la lunghezza del file di log - controllo limiti ???
		
		IGBuilderUpdated igbuilder = new IGBuilderUpdated(log, db, conformanceResult, repFlag);
		igbuilder.createIGSet();
		model=igbuilder.getGraphsSet();
		ModelResult mr = new ModelResult(model);
		mr.setRes(igbuilder.getGraphsSet());

		mr.setLog(log);
		return mr;
		// return "hello world";
	}
	
	/*
	private static void mapEventsToTransition(TransEvClassMapping mapping, PetrinetImpl petriNet, XEventClass dummy) 
	{
		Collection<Transition> transitionList=petriNet.getTransitions();
		Collection<XEventClass> eventClasses=this.getEventClasses();
		ArrayList<Transition> unmapped= new ArrayList<Transition>();
		for(Transition t: transitionList){
			boolean mapped=false;
			String tName=t.getLabel();
			for(XEventClass evClass: eventClasses){
				if(evClass.getId() != null && evClass.getId().equals(tName))
				{
					mapping.put(t, evClass);
					mapped=true;
					break;
				}
			}
			if(!mapped)
				unmapped.add(t);
		}
		for(Transition t: unmapped){
			mapping.put(t, dummy);
		}
		
	}
*/
	private static void updateCasualRel(ArrayList<ArrayList<String>> table,
			DBManager dbManager) {

		String queryInsert = "INSERT INTO causalrel(source,target) VALUES(?,?)";

		for (int i = 0; i < table.size(); i++) {
			dbManager.executeStatement(queryInsert, table.get(i));
		}
	}

	private static ArrayList<AlignedTrace> updateAlignment(
			ArrayList<ArrayList<String>> table, DBManager dbManager) {
		ArrayList<AlignedTrace> alignedTraces = new ArrayList<AlignedTrace>();
		String queryInsert = "INSERT INTO alignments(traceId,alignment) VALUES(?,?)";

		for (int i = 0; i < table.size(); i++) {
			dbManager.executeStatement(queryInsert, table.get(i));
			String traceId = table.get(i).get(0);
			String alignment = table.get(i).get(1);
			updateAlignedEvents(alignment, traceId, dbManager);
			if (alignment.contains("[L]") || alignment.contains("[M-REAL]")) {

				AlignedTrace alignedTrace = new AlignedTrace(traceId, alignment);
				alignedTraces.add(alignedTrace);

			}
		}
		return alignedTraces;
	}

	private static HashMap<Integer, ArrayList<Event>> updateEventTable(
			XLog log, DBManager dbManager) {
		HashMap<Integer, ArrayList<Event>> result = new HashMap<Integer, ArrayList<Event>>();
		int numTrace = 1;
		for (XTrace trace : log) {
			int posEv = 0;
			ArrayList<Event> events = new ArrayList<Event>();
			for (XEvent event : trace) {
				posEv++;
				String eventName = event.getAttributes().get("concept:name")
						.toString();
				String eventClass = eventName;
				if (event.getAttributes().get("lifecycle:transition") != null) {
					String eventType = event.getAttributes()
							.get("lifecycle:transition").toString();
					eventClass = eventClass + "+" + eventType;
				}
				eventClass = eventClass.replace(" ", "");
				String queryInsert = "INSERT INTO event(posTrace,class,trace) VALUES(?,?,?)";
				ArrayList<String> fieldList = new ArrayList<String>(
						Arrays.asList(String.valueOf(posEv), eventClass,
								String.valueOf(numTrace)));
				dbManager.executeStatement(queryInsert, fieldList);
				Event eventTrace = new Event(String.valueOf(posEv), eventClass);
				events.add(eventTrace);
			}
			result.put(numTrace, events);
			numTrace++;
		}
		return result;
	}

	private static Object generateMatchString(SyncReplayResult res) {
		StringBuilder sb = new StringBuilder();
		String limiter = "";
		List<Object> nodeInstance = res.getNodeInstance();

		int counter = 0;
		int size = res.getTraceIndex().size();
		for (StepTypes stepType : res.getStepTypes()) {
			sb.append(limiter);
			switch (stepType) {
			case L:
				sb.append("[L]");
				sb.append(nodeInstance.get(counter));
				break;

			case MINVI:
				sb.append("[M-INVI]");
				sb.append(((Transition) nodeInstance.get(counter)).getLabel());
				break;

			case MREAL:
				sb.append("[M-REAL]");
				sb.append(((Transition) nodeInstance.get(counter)).getLabel());
				break;

			case LMNOGOOD:
				sb.append("[L/M-NOGOOD]");
				sb.append(((Transition) nodeInstance.get(counter)).getLabel());
				break;

			case LMREPLACED:
				sb.append("[L/M-REPLACED]");
				ViolatingSyncMove violation = (ViolatingSyncMove) nodeInstance
						.get(counter);
				sb.append("Trans:" + violation.getTransition() + "-->"
						+ violation.getEventClass().getId());
				break;

			case LMSWAPPED:
				sb.append("[L/M-SWAPPED]");
				SwappedMove swapMove = (SwappedMove) nodeInstance.get(counter);
				sb.append("Trans:" + swapMove.getOccurTrans().getLabel()
						+ " insteadOf " + swapMove.getInsteadOf().getLabel());
				break;

			default:
				sb.append("[L/M]");
				sb.append(((Transition) nodeInstance.get(counter)).getLabel());
			}
			limiter = "|";
			counter++;
		}
		return sb.toString();
	}

	private static void deleteTauFromRel(int from, int to, DBManager dbManager) {
		//
		String querySelect = "select  source,target " + "FROM causalrel";
		ArrayList<String> rowList = dbManager.executeQuery(querySelect,
				"source;target");
		int flag = 0;
		// for(int numRow=from; numRow<to; numRow++){
		for (String row : rowList) {
			// String row=rowList.get(numRow);
			flag++;
			System.out.println(flag);
			ArrayList<String> visited = new ArrayList<String>();
			String[] token = row.split(";");
			String sourceName = token[0];
			String targetName = token[1];
			if (!visited.contains(sourceName + ";" + targetName)) {
				if (!sourceName.contains("tauINVISIBLE") && !targetName.contains("tauINVISIBLE")) {
					// String
					// queryUpdate="UPDATE edge SET visited=1 WHERE source LIKE'"+sourceId+"' AND "
					// + " target LIKE '"+targetId+"'";
					// dbManager.queryUpdate(queryUpdate);
				} else {
					String tauTran = "";
					int type = -1;
					if (targetName.contains("tauINVISIBLE")) {
						tauTran = targetName;
						type = 0;
					} else if (sourceName.contains("tauINVISIBLE")) {
						tauTran = sourceName;
						type = 1;
					}
					String newTran = "";
					if (type != 1) {
						newTran = findTransition(tauTran, type, visited,
								dbManager);
						System.out.println("tran");
					}
					// should be optimized, I only have to check if the tau is a
					// target
					String[] tokenTran = newTran.split(";");
					if (tokenTran.length > 0) {
						for (int i = 0; i < tokenTran.length; i++) {
							if (!tokenTran[i].equals("")) {
								// 18-12 aggiungi un controllo per gli eventuali
								// duplicati
								String queryInsert = "INSERT INTO causalrel(source,target) VALUES(?,?)";
								String newSource = "";
								String newTarget = "";
								if (type == 0) {
									// the target has been changed
									newTarget = tokenTran[i];
									newSource = sourceName;
								} else if (type == 1) {
									// the source has been changed
									newTarget = targetName;
									newSource = tokenTran[i];
								}
								ArrayList<String> fieldQuery = new ArrayList<String>(
										Arrays.asList(newSource, newTarget));
								// check if the relation already exists
								String queryCheck = "SELECT source,target FROM causalrel WHERE source LIKE '"
										+ newSource
										+ "' AND target LIKE '"
										+ newTarget + "'";
								ArrayList<String> checkResult = dbManager
										.executeQuery(queryCheck,
												"source;target");
								if (checkResult.size() == 0)
									dbManager.executeStatement(queryInsert,
											fieldQuery);
							}
						}
					}
				}
			}
		}
		// At the end we remove from causalrel the tau causal rel.
		String queryRemoveTau = "DELETE FROM causalrel WHERE source LIKE '%tauINVISIBLE%' OR target LIKE '%tauINVISIBLE%'";
		dbManager.queryUpdate(queryRemoveTau);
	}

	private static String findTransition(String candidateId, int type,
			ArrayList<String> visited, DBManager dbManager) {
		String resultTran = "";
		int newType = -1;
		if (!candidateId.contains("tauINVISIBLE"))
			resultTran = candidateId;
		else {
			String where = " WHERE";
			String field = "";
			if (type == 0) {
				where = where + " source LIKE '" + candidateId + "'";
				field = "target";
				newType = 0;
			} else if (type == 1) {
				where = where + " target LIKE '" + candidateId + "'";
				field = "source";
				newType = 0;
			}
			String queryNewCandidate = "SELECT " + field + " FROM causalrel "
					+ where;
			ArrayList<String> candList = dbManager.executeQuery(
					queryNewCandidate, field);
			if (candList.size() > 0) {
				for (String cand : candList) {
					// if the rel has been already visited or if the source has
					// been already considered, skip
					if (visited.contains(candidateId + "," + cand)
							|| alreadyChecked(visited, cand)) {
						resultTran = resultTran + "";
					}

					else {
						String oldSource = "";
						String oldTarget = "";
						if (type == 0) {
							oldSource = candidateId;
							oldTarget = cand;
						} else if (type == 1) {
							oldSource = cand;
							oldTarget = candidateId;
						}
						visited.add(oldSource + ";" + oldTarget);
						// String
						// queryDelete="DELETE FROM causalrel WHERE source LIKE '"+oldSource+"' AND target LIKE '"+oldTarget+"'";
						// dbManager.queryUpdate(queryDelete);
						resultTran = resultTran
								+ ";"
								+ findTransition(cand, newType, visited,
										dbManager);

					}
				}
			}
		}
		return resultTran;
	}

	private static boolean alreadyChecked(ArrayList<String> visited, String cand) {
		boolean result = false;
		for (String edge : visited) {
			String[] token = edge.split(";");
			if (token.length > 0) {
				String source = token[0];
				if (source.equals(cand)) {
					result = true;
					break;
				}
			}
		}
		return result;
	}

	private static void updateAlignedEvents(String alignment,
			String traceIdList, DBManager dbManager) {
		String[] tokenTrace = traceIdList.replaceAll("\"", "").split("\\|");
		String[] tokensEventWithAlignment = alignment.split("\\|");
		HashMap<Integer, String> posEvents = new HashMap<Integer, String>();
		HashMap<Integer,String> kindAlignmentEvents= new HashMap<Integer, String>();
		int posEvent = 1;
		for (int j = 0; j < tokensEventWithAlignment.length; j++) {
			String[] tokensEvent = tokensEventWithAlignment[j].split("]");
			String eventClass = tokensEvent[1];
			String anomalyKind= tokensEvent[0];
			if(!anomalyKind.contains("M-INVI")){
				posEvents.put(posEvent, eventClass);
				kindAlignmentEvents.put(posEvent, tokensEvent[0]);
				posEvent++;
			}
		}
		for (int i = 0; i < tokenTrace.length; i++) {
			String traceId = tokenTrace[i];
			for (int pos : posEvents.keySet()) {
				//update 14/06
				/*String queryInsert = "INSERT INTO alignedevent(posTrace,class,trace,traceid,timestamp,kind) "
						+ " VALUES('"
						+ pos
						+ "','"
						+ posEvents.get(pos)
						+ "',0,'" + traceId + "',NULL)";
				dbManager.queryUpdate(queryInsert);*/
				String queryInsert = "INSERT IGNORE INTO alignedevent(posTrace,class,trace,traceid,"
						+ "timestamp,kind)"
						+ " VALUES(?,?,?,?,?,?)";
				//remove space and ' char
				String classEventNoSpace=posEvents.get(pos).replace(" ", "");
				classEventNoSpace=classEventNoSpace.replace("'", "");
				String kind=kindAlignmentEvents.get(pos);
				ArrayList<String> fieldList = new ArrayList<String>(
					Arrays.asList(String.valueOf(pos), classEventNoSpace, "0", traceId, "NULL",kind));
				dbManager.executeStatement(queryInsert, fieldList);
			}
		}
	}

	public static IGConfiguration setConfig(final UIPluginContext context, final IGConfiguration config, int variant) {
		ProMPropertiesPanel panel = new ProMPropertiesPanel("Configure IG Building");
		//JCheckBox checkBox = panel.addCheckBox("Repair");
		//checkBox.setSelected(true);
		//boolean rep = true;
		
		// variante 1 - solo LOG
		if (variant == 1) 
		{
			JCheckBox checkBox = panel.addCheckBox("Repair");
			checkBox.setSelected(true);
			boolean rep = true;
			
			String[] optItems = { "Alpha", "Heuristic", "Inductive" };
			final JComboBox optList = panel.addComboBox("Process discovery", optItems);
			optList.setSelectedItem("Inductive");
			// default choice
			selectedPrcessDiscoveryMode = "Inductive";
			config.setpDiscovery(selectedPrcessDiscoveryMode);
			//System.out.println("INIT: " + selectedPrcessDiscoveryMode);

			optList.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					selectedPrcessDiscoveryMode = optList.getSelectedItem().toString();
					config.setpDiscovery(selectedPrcessDiscoveryMode);
					
					// System.out.println(optList.getSelectedItem().toString());
					// System.out.println(selectedPrcessDiscoveryMode);
				}
			});
			
			ItemListener checkBoxListener = new CheckBoxListener(config);
			checkBox.addItemListener(checkBoxListener);
			final InteractionResult interactionResult = context.showConfiguration(
					"Setup IG Building", panel);
			if (interactionResult == InteractionResult.FINISHED
					|| interactionResult == InteractionResult.CONTINUE
					|| interactionResult == InteractionResult.NEXT) {
				return config;
			}

		}
		
		// variante 2 - LOG + altri parametri
		if (variant == 2 && checkCaller == false)
		{
			JCheckBox checkBox = panel.addCheckBox("Repair");
			checkBox.setSelected(true);
			boolean rep = true;
			
			ItemListener checkBoxListener = new CheckBoxListener(config);
			checkBox.addItemListener(checkBoxListener);
			final InteractionResult interactionResult = context.showConfiguration(
					"Setup IG Building", panel);
			if (interactionResult == InteractionResult.FINISHED
					|| interactionResult == InteractionResult.CONTINUE
					|| interactionResult == InteractionResult.NEXT) {
				return config;
			}
		}
		
		/*
		ItemListener checkBoxListener = new CheckBoxListener(config);
		checkBox.addItemListener(checkBoxListener);
		final InteractionResult interactionResult = context.showConfiguration(
				"Setup IG Building", panel);
		if (interactionResult == InteractionResult.FINISHED
				|| interactionResult == InteractionResult.CONTINUE
				|| interactionResult == InteractionResult.NEXT) {
			return config;
		}
		*/
		return null;
	}
	
	private static XLog manipulateEventLog(XLog xlog) {
		//simply remove the lifecycle attribute	
		int numTrace=1;
		Iterator<XTrace> it= xlog.iterator();
		XLogImpl xlog2=new XLogImpl(xlog.getAttributes());
		while(it.hasNext()){
			XTrace trace=it.next();
			Iterator<XEvent> eventsIterator=trace.iterator();
			while(eventsIterator.hasNext()){
				XEvent event=eventsIterator.next();
				XAttributeMap attMap=event.getAttributes();
				if(attMap.containsKey("lifecycle:transition")){
					attMap.remove("lifecycle:transition");
				}
				//update 19/06; remove possible , and '
				if(attMap.containsKey("concept:name")){
					String name=attMap.get("concept:name").toString();
					name=name.replace(",", "");
					name=name.replace("'","");
					attMap.remove("concept:name");
					XAttributeLiteralImpl newName= new XAttributeLiteralImpl("concept:name",name);
					attMap.put("concept:name", newName);
				}
			}
			xlog2.add(trace);
			
		}
		//write the new log
				OutputStream output=null;
				try {
					output = new FileOutputStream("preprocessedLog.xes");
				} catch (FileNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				XesXmlSerializer serializer= new XesXmlSerializer() ;
				try {
					serializer.serialize(xlog2, output);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				return xlog2;
	}

}