package org.processmining.big4prom.plugins;

import instancegraph.models.InstanceGraphModel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.jgrapht.DirectedGraph;
import org.processmining.big4prom.models.ModelResult;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.jgraph.ProMGraphModel;
import org.processmining.models.jgraph.ProMJGraph;

import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.JGraphLayout;
import com.jgraph.layout.hierarchical.JGraphHierarchicalLayout;
// import org.processmining.plugins.log.ui.logdialog.ProcessInstanceView;
// import org.processmining.plugins.log.ui.logdialog.InstanceCellRenderer;
// import org.processmining.plugins.log.ui.logdialog.XTraceListModel;
// import javafx.scene.layout.Border;

@Plugin(
		name = "Show Instance Graph", 
		returnLabels = { "Visualization of Instance Graph Model" }, 
		returnTypes = { JComponent.class }, 
		parameterLabels = { "Instance Graphs Set" }, 
		userAccessible = true
	)
	@Visualizer
public class InstanceGraphVisualizer
{
	JPanel container = new JPanel();
	JPanel graphPanel = new JPanel();
	
	JList<Integer> list;
	DefaultListModel<Integer> listModel;
	
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, ModelResult res) 
	{								
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		
		// container.setLayout(new WrapLayout());
		// container.setLayout(new GridLayout(2,1));
		container.setLayout(new BoxLayout(container, BoxLayout.X_AXIS));
		
		// graphPanel.setLayout(new WrapLayout());
		graphPanel.setLayout(new BoxLayout(graphPanel, BoxLayout.X_AXIS));
		graphPanel.setSize(screenSize.width - 50, screenSize.height - 50);
				
		listModel = new DefaultListModel<Integer>();
		Set<Integer> graphs = res.getRes().keySet();
		for(Integer i : graphs){
			listModel.addElement(i);
		}
		
		list = new JList<Integer>(listModel);
		list.setFixedCellWidth(50);
	    list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    list.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
	       
		list.addListSelectionListener(new ListSelectionListener() {
			ModelResult mr = null;
		      public void valueChanged(ListSelectionEvent evt) {
		    	  Integer idx = list.getSelectedIndex() + 1;
		    	  
		    	  _renderGraph(mr, graphPanel, idx);
		    	  graphPanel.repaint();
		    	  graphPanel.revalidate();
		      }
		      
		      private ListSelectionListener init(ModelResult var){
			      mr = var;
			      return this;
			  }
		    }.init(res));
		
		container.add(new JScrollPane(list));
		container.add(graphPanel);
		
		JScrollPane scrollPane = new JScrollPane(container); 
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		
		// inizialmente visualizzo il primo grafo presente nella lista
		_renderGraph(res, graphPanel, listModel.getElementAt(0));
		graphPanel.repaint();
		graphPanel.revalidate();
			
		return scrollPane;
	}

	private void _renderGraph(
			ModelResult graphModel, 
			JPanel container,
			int choice)
	{
	
			HashMap<Integer, DirectedGraph<String, org.jgrapht.graph.DefaultEdge>> instanceGraphList=graphModel.getRes();
				ProMJGraph graph = _factoryGraphPanel(instanceGraphList, choice);
								
				JScrollPane panel = new JScrollPane();
				panel.setViewportView(graph);
				
				// la dimensione � calcolata a partendo dalla risoluzione dello schermo
				Dimension dim = new Dimension();
				Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

				dim.setSize(screenSize.width - 100, screenSize.height - 200);
				
				panel.setPreferredSize(dim);
				// dimensione fissa
				// graphPanel.setPreferredSize(new Dimension(800, 600));
				
				panel.setMaximumSize(panel.getPreferredSize());
				panel.setMinimumSize(panel.getPreferredSize());
				
				// rimuovo tutto
				container.removeAll();
				
				// aggiungo nuovamente combobox e grafo
				// container.add(graphSelect); /* vecchia interfaccia */
				container.add(new JScrollPane(list));
				container.add(panel);
				
				graph.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
				graph.setBackground(Color.white);
				panel.setBackground(Color.white);
	}

	private JLabel _factoryLabel(String text)
	{
		JLabel label = new JLabel(text);
		label.setBackground(Color.white);
		return label;
	}

	private ProMJGraph _factoryGraphPanel(HashMap<Integer, DirectedGraph<String, org.jgrapht.graph.DefaultEdge>> instanceGraphList
			,int choice)
	{
		DirectedGraph<String, org.jgrapht.graph.DefaultEdge>  instanceGraph= instanceGraphList.get(choice);
		return _factoryGraphPanel(new InstanceGraphModel(instanceGraph));
	}

	
	private ProMJGraph _factoryGraphPanel(InstanceGraphModel graph)
	{
		// Borrowed from ProMJGraphVisualizer
		GraphLayoutConnection layoutConnection = new GraphLayoutConnection(graph);
		ProMGraphModel gModel = new ProMGraphModel(graph);
		ViewSpecificAttributeMap map = new ViewSpecificAttributeMap();
		ProMJGraph jgraph = new ProMJGraph(gModel, map, layoutConnection);
		
		JGraphLayout layout = getLayout(map.get(graph, AttributeMap.PREF_ORIENTATION, SwingConstants.SOUTH));
		if (!layoutConnection.isLayedOut())
		{
			JGraphFacade facade = new JGraphFacade(jgraph);

			facade.setOrdered(false);
			facade.setEdgePromotion(true);
			facade.setIgnoresCellsInGroups(false);
			facade.setIgnoresHiddenCells(false);
			facade.setIgnoresUnconnectedCells(false);
			facade.setDirected(true);
			facade.resetControlPoints();
			if (layout instanceof JGraphHierarchicalLayout) {
				facade.run((JGraphHierarchicalLayout) layout, true);
			} else {
				facade.run(layout, true);
			}

			Map<?, ?> nested = facade.createNestedMap(true, true);

			jgraph.getGraphLayoutCache().edit(nested);
			layoutConnection.setLayedOut(true);

		}

		jgraph.setUpdateLayout(layout);
		layoutConnection.updated();
		return jgraph;
	}
	
	protected JGraphLayout getLayout(int orientation)
	{
		// Borrowed from ProMJGraphVisualizer
		JGraphHierarchicalLayout layout = new JGraphHierarchicalLayout();
		layout.setDeterministic(false);
		layout.setCompactLayout(true);
		layout.setFineTuning(true);
		layout.setParallelEdgeSpacing(15);
		layout.setFixRoots(false);

		layout.setOrientation(orientation);

		return layout;
	}
	
}
